@@ -3,8 +3,6 @@
 
 #include <stdbool.h>
 
-#include "args.h"
-
 // medaka-style feature data
 typedef struct _plp_data {
     size_t buffer_cols;
@@ -99,20 +97,8 @@ void print_bedmethyl(plp_data pileup, char *ref, int rstart, bool extended, char
  *
  */
 plp_data calculate_pileup(
-    const char **bam_file, const char *chr, int start, int end,
+    const set_fsets *fsets, const char *chr, int start, int end,
     const char *read_group, const char tag_name[2], const int tag_value,
     int lowthreshold, int highthreshold, char mod_base);
 
-
-/* Process and print a single region using a threadpool
- *
- * @param args program arguments.
- * @param chr reference sequence to process.
- * @param start reference coordinate to process (0-based).
- * @param end reference coordiate to process (exclusive).
- * @param ref reference sequence.
- *
- */
-void process_region(arguments_t args, const char *chr, int start, int end, char *ref);
-
 #endif
